--
--Canon
-- Specialization for Irrigator
--
-- @author  Nico0179
-- free for noncommerical-usage
--
 source("irrigator.lua");

Canon = {};

function Canon.prerequisitesPresent(specializations)
	return true;
end;

function Canon:load(xmlFile)
	
	self.setChopperWidthState = SpecializationUtil.callSpecializationsFunction("setChopperWidthState");
	self.getNewChopperRot = Canon.getNewChopperRot;
	

	self.chopperRotatingParts = {};
	self.chopperRotatingParts.chopperParts = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.chopperRotatingParts.chopperPart(%d)", i);
		local node = getXMLString(xmlFile, key .. "#node");
		if node == nil then
			break;
		end;
		local part = {};
		part.node = Utils.indexToObject(self.components, node);
		part.inverse = Utils.getNoNil(getXMLBool(xmlFile, key .. "#inverse"), false);
		part.states = {};
		part.currentMinRot = nil;
		part.currentMaxRot = nil;
		local j = 0;
		while true do
			local stateKey = string.format(key .. ".state(%d)", j);
			local minRot = getXMLFloat(xmlFile, stateKey .. "#minRot");
			if minRot == nil then
				break;
			end;
			local state = {};
			state.minRot = {math.rad(minRot)};
			if part.currentMinRot == nil then
				part.currentMinRot = {state.minRot[1]};
			end;
			state.maxRot = {math.rad(getXMLInt(xmlFile, stateKey .. "#maxRot"))};
			
			if part.currentMaxRot == nil then
				part.currentMaxRot = {state.maxRot[1]};
			end;
			state.speed = getXMLFloat(xmlFile, stateKey .. "#speed") * 1000;
			state.i18n = getXMLString(xmlFile, stateKey .. "#i18n");
			table.insert(part.states, state);
			j = j + 1;				
		end;		
		table.insert(self.chopperRotatingParts.chopperParts, part);
		i = i + 1;
	end;	
	self.radsPerMS = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.chopperRotatingParts#degreePerSecond"), 5) / 1000);
	self.chopperWidthState = 0;
	self.currentChopperDirection = -1;
	self.numChopperWidthStates = table.getn(self.chopperRotatingParts.chopperParts[1].states);
	self.chopperLeftDirectionTime = self.chopperRotatingParts.chopperParts[1].states[1].speed;
	
	
	self.chopperLeftDirection = false;		
	self.isSelectable = true;
end;

function Canon:delete()
end;

function Canon:readStream(streamId, connection)
	local state = streamReadInt8(streamId);
	self:setChopperWidthState(state, true);
end;

function Canon:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.chopperWidthState);
end;

function Canon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Canon:keyEvent(unicode, sym, modifier, isDown)
end;

function Canon:update(dt)
	if self:getIsActive() then
				if InputBinding.hasEvent(InputBinding.SWITCH_CHOPPER_WIDTH) then			
					self:setChopperWidthState((self.chopperWidthState + 1) % (self.numChopperWidthStates));		
				end;
	end;
end;

function Canon:updateTick(dt)

     if self.isTurnedOn and self.allowsSpraying then
	
			self.chopperLeftDirectionTime = self.chopperLeftDirectionTime - dt;
					
			for k, chopperPart in pairs(self.chopperRotatingParts.chopperParts) do
				local state = chopperPart.states[self.chopperWidthState+1];
				local x, y, z = getRotation(chopperPart.node);
				local dir = self.chopperLeftDirection;
				if chopperPart.inverse then
					dir = not self.chopperLeftDirection;
				end;
				local newRot = unpack(Utils.getMovedLimitedValues({y}, chopperPart.currentMaxRot, chopperPart.currentMinRot, 1, state.speed, dt, dir));
				setRotation(chopperPart.node, x, newRot, z);				
				chopperPart.currentMinRot[1] = self:getNewChopperRot(chopperPart.currentMinRot[1], state.minRot[1], dt);
				chopperPart.currentMaxRot[1] = self:getNewChopperRot(chopperPart.currentMaxRot[1], state.maxRot[1], dt);
				if k == 1 then
					if self.chopperLeftDirectionTime < 0 then
						self.chopperLeftDirection = not self.chopperLeftDirection;
						self.chopperLeftDirectionTime = state.speed;
					end;
				end;
			end;

	end;	
end;

function Canon:draw()	
	if self:getIsActive() then
				local width = self.chopperRotatingParts.chopperParts[1].states[self.chopperWidthState+1].i18n;
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("SWITCH_CHOPPER_WIDTH_TEXT"), g_i18n:getText(width)), InputBinding.SWITCH_CHOPPER_WIDTH);
	end;
end;

function Canon:getNewChopperRot(currentRot, newRot, dt)
	if newRot < currentRot then
		return math.max(newRot, currentRot - (dt * self.radsPerMS));
	else
		return math.min(newRot, currentRot + (dt * self.radsPerMS));
	end;
end;

function Canon:setChopperWidthState(state, noEventSend)
	ChopperWidthEvent.sendEvent(self, state, noEventSend);	
	self.chopperWidthState = math.max(0, math.min(self.numChopperWidthStates - 1, state));
end;


function Canon:onDeactivate()
end;

--
-- ChopperWidthEvent
-- Networkevent for Irrigator
--
ChopperWidthEvent = {};
ChopperWidthEvent_mt = Class(ChopperWidthEvent, Event);

InitEventClass(ChopperWidthEvent, "ChopperWidthEvent");

function ChopperWidthEvent:emptyNew()
    local self = Event:new(ChopperWidthEvent_mt);
    return self;
end;

function ChopperWidthEvent:new(vehicle, state)
    local self = ChopperWidthEvent:emptyNew()
    self.vehicle = vehicle;
	self.state = state;
    return self;
end;

function ChopperWidthEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.vehicle = networkGetObject(id);
	self.state = streamReadInt8(streamId);
    self:run(connection);
end;

function ChopperWidthEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteInt8(streamId, self.state);
end;

function ChopperWidthEvent:run(connection)
	self.vehicle:setChopperWidthState(self.state, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(ChopperWidthEvent:new(self.vehicle, self.state), nil, connection, self.vehicle);
	end;
end;

function ChopperWidthEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ChopperWidthEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ChopperWidthEvent:new(vehicle, state));
		end;
	end;
end;
